@@ -1,5 +1,5 @@
 # This file is part of Pebble.
-# Copyright (c) 2013-2022, Matteo Cafasso
+# Copyright (c) 2013-2023, Matteo Cafasso
 
 # Pebble is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License
@@ -24,7 +24,7 @@ from typing import Any, Callable
 _synchronized_lock = threading.Lock()
 
 
-def synchronized(*args):
+def synchronized(*args) -> Callable:
     """A synchronized function prevents two or more callers to interleave
     its execution preventing race conditions.
 
@@ -39,7 +39,7 @@ def synchronized(*args):
     if callable(args[0]):
         return decorate_synchronized(args[0], _synchronized_lock)
     else:
-        def wrap(function):
+        def wrap(function) -> type:
             return decorate_synchronized(function, args[0])
 
         return wrap
